@@ -279,6 +279,7 @@ sub extract_tagged (;$$$$$) # ($text, $opentag, $closetag, $pre, \%options)
 sub _match_tagged	# ($$$$$$$)
 {
 	my ($textref, $pre, $ldel, $rdel, $omode, $bad, $ignore) = @_;
+	my $rdelspec;
 
 	my ($startpos, $opentagpos, $textpos, $parapos, $closetagpos, $endpos) = ( pos($$textref) = pos($$textref)||0 );
 
@@ -290,7 +291,7 @@ sub _match_tagged	# ($$$$$$$)
 
 	$opentagpos = pos($$textref);
 
-	unless ($$textref =~ m/\G($ldel)/gc)
+	unless ($$textref =~ m/\G$ldel/gc)
 	{
 		$@ = "Did not find opening tag: /$ldel/";
 		goto failed;
@@ -300,13 +301,17 @@ sub _match_tagged	# ($$$$$$$)
 
 	if (!defined $rdel)
 	{
-		$rdel = $1;
-		unless ($rdel =~ s/\A([[(<{]+)($XMLNAME).*/ "$1\/$2". revbracket($1) /oes)
+		$rdelspec = $&;
+		unless ($rdelspec =~ s/\A([[(<{]+)($XMLNAME).*/ "$1\/$2". revbracket($1) /oes)
 		{
 			$@ = "Unable to construct closing tag to match: $rdel";
 			goto failed;
 		}
 	}
+	else
+	{
+		$rdelspec = eval "qq{$rdel}";
+	}
 
 	while (pos($$textref) < length($$textref))
 	{
@@ -317,7 +322,7 @@ sub _match_tagged	# ($$$$$$$)
 			$parapos = pos($$textref) - length($1)
 				unless defined $parapos;
 		}
-		elsif ($$textref =~ m/\G($rdel)/gc )
+		elsif ($$textref =~ m/\G($rdelspec)/gc )
 		{
 			$closetagpos = pos($$textref)-length($1);
 			goto matched;
@@ -692,7 +697,8 @@ sub _match_quotelike($$$$)	# ($textref, $prepat, $allow_raw_match)
 	}
 	$ld2pos = $rd1pos = pos($$textref)-1;
 
-	if ($op =~ /s|tr|y/)
+	my $second_arg = $op =~ /s|tr|y/ ? 1 : 0;
+	if ($second_arg)
 	{
 		my ($ldel2, $rdel2);
 		if ($ldel1 =~ /[[(<{]/)
@@ -741,9 +747,9 @@ sub _match_quotelike($$$$)	# ($textref, $prepat, $allow_raw_match)
 		$ld1pos,	1,			# LEFT DEL
 		$str1pos,	$rd1pos-$str1pos,	# STR/PAT
 		$rd1pos,	1,			# RIGHT DEL
-		$ld2pos,	1,			# 2ND LDEL
-		$str2pos,	$rd2pos-$str2pos,	# 2ND STR
-		$rd2pos,	1,			# 2ND RDEL
+		$ld2pos,	$second_arg,		# 2ND LDEL (MAYBE)
+		$str2pos,	$rd2pos-$str2pos,	# 2ND STR (MAYBE)
+		$rd2pos,	$second_arg,		# 2ND RDEL (MAYBE)
 		$modpos,	$endpos-$modpos,	# MODIFIERS
 		$endpos,	$textlen-$endpos,	# REMAINDER
 	       );
@@ -781,13 +787,29 @@ sub extract_multiple (;$$$$)	# ($text, $functions_ref, $max_fields, $ignoreunkno
 
 		my $unkpos;
 		my $func;
-		my $field;
+		my $class;
+
+		my @class;
+		foreach $func ( @func )
+		{
+			if (ref($func) eq 'HASH')
+			{
+				push @class, (keys %$func)[0];
+				$func = (values %$func)[0];
+			}
+			else
+			{
+				push @class, undef;
+			}
+		}
 
 		FIELD: while (pos() < length())
 		{
-			foreach $func ( @func )
+			my $field;
+			foreach my $i ( 0..$#func )
 			{
-				undef $field;
+				$func = $func[$i];
+				$class = $class[$i];
 				$lastpos = pos;
 				if (ref($func) eq 'CODE')
 					{ ($field) = $func->($_) }
@@ -803,7 +825,9 @@ sub extract_multiple (;$$$$)	# ($text, $functions_ref, $max_fields, $ignoreunkno
 						undef $unkpos;
 						last FIELD if @fields == $max;
 					}
-					push @fields, $field;
+					push @fields, $class 
+						? bless(\$field, $class)
+						: $field;
 					$firstpos = $lastpos unless defined $firstpos;
 					$lastpos = pos;
 					last FIELD if @fields == $max;
